@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000, 2001, 2013 Gregory Trubetskoy
+ * Copyright (C) 2000, 2001, 2013, 2024 Gregory Trubetskoy
  * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Apache Software Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License"); you
@@ -69,12 +69,6 @@ static PyObject * make_obcallback(const char *name)
     PyObject *m = NULL;
     PyObject *obCallBack = NULL;
 
-    /* This makes _apache appear imported, and subsequent
-     * >>> import _apache
-     * will not give an error.
-     */
-    _apache_module_init();
-
     /* Now execute the equivalent of
      * >>> import <module>
      * >>> <initstring>
@@ -303,7 +297,7 @@ static void release_interpreter(interpreterdata *idata)
 {
     PyThreadState *tstate = PyThreadState_Get();
 #ifdef WITH_THREAD
-#if PY_MAJOR_VERSION <= 3 && PY_MINOR_VERSION < 10 
+#if PY_MAJOR_VERSION <= 3 && PY_MINOR_VERSION < 9
     PyThreadState_Clear(tstate);
 #endif
     if (idata)
@@ -710,7 +704,6 @@ static int python_init(apr_pool_t *p, apr_pool_t *ptemp,
     const char *userdata_key = "python_init";
     apr_status_t rc;
 
-    const char *py_compile_version = PY_VERSION;
     const char *py_dynamic_version = 0;
 
     /* The "initialized" flag is a fudge for Mac OS X. It
@@ -752,19 +745,7 @@ static int python_init(apr_pool_t *p, apr_pool_t *ptemp,
     /* mod_python version */
     ap_add_version_component(p, mp_version_component);
 
-    py_dynamic_version = strtok((char *)Py_GetVersion(), " ");
-
-    if (strcmp(py_compile_version, py_dynamic_version) != 0) {
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, s,
-                     "python_init: Python version mismatch, expected '%s', found '%s'.",
-                     py_compile_version, py_dynamic_version);
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, s,
-                     "python_init: Python executable found '" PRIs "'.",
-                     Py_GetProgramFullPath());
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, s,
-                     "python_init: Python path being used '" PRIs "'.",
-                     Py_GetPath());
-    }
+    py_dynamic_version = strtok(apr_pstrdup(p, Py_GetVersion()), " ");
 
     /* Python version */
     sprintf(buff, "Python/%.200s", py_dynamic_version);
@@ -783,10 +764,10 @@ static int python_init(apr_pool_t *p, apr_pool_t *ptemp,
     if (initialized == 0 || !Py_IsInitialized())
     {
         initialized = 1;
-
+#if PY_VERSION_HEX < 0x030C0000
         /* disable user site directories */
         Py_NoUserSiteDirectory = 1;
-
+#endif
         /* Initialze the main interpreter. */
 #if PY_MAJOR_VERSION == 2 && \
                 (PY_MINOR_VERSION < 7 || (PY_MINOR_VERSION == 7 && PY_MICRO_VERSION < 14))
@@ -800,6 +781,14 @@ static int python_init(apr_pool_t *p, apr_pool_t *ptemp,
         Py_NoSiteFlag = 1;
 #endif
 
+#if PY_MAJOR_VERSION == 2
+        PyMODINIT_FUNC init_apache(void);
+        PyImport_AppendInittab("_apache", &init_apache);
+#else
+        PyMODINIT_FUNC PyInit_apache(void);
+        PyImport_AppendInittab("_apache", &PyInit_apache);
+#endif
+
         Py_Initialize();
 
 #if PY_MAJOR_VERSION == 2 && \
@@ -807,7 +796,8 @@ static int python_init(apr_pool_t *p, apr_pool_t *ptemp,
         Py_NoSiteFlag = 0;
 #endif
 
-#ifdef WITH_THREAD
+        /* see https://docs.python.org/3/c-api/init.html#c.PyEval_InitThreads */
+#if defined (WITH_THREAD) && PY_VERSION_HEX < 0x03070000
         /* create and acquire the interpreter lock */
         PyEval_InitThreads();
 #endif
@@ -1977,6 +1967,25 @@ static apr_status_t handle_python(include_ctx_t *ctx,
         return APR_SUCCESS;
     }
 
+    /* get configuration */
+    conf = (py_config *) ap_get_module_config(req->per_dir_config,
+                                              &python_module);
+
+    /* determine interpreter to use */
+    interp_name = select_interp_name(req, NULL, conf, NULL, NULL);
+
+    /* get/create interpreter */
+    idata = get_interpreter(interp_name);
+
+    if (!idata) {
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, req,
+                      "handle_python: Can't get/create interpreter.");
+
+        Py_XDECREF(tagobject);
+        Py_XDECREF(codeobject);
+        return HTTP_INTERNAL_SERVER_ERROR;
+    }
+
     /* process tags */
     while (1) {
         optfn_ssi_get_tag_and_value(ctx, &tag, &tag_val, 1);
@@ -2030,25 +2039,6 @@ static apr_status_t handle_python(include_ctx_t *ctx,
         return APR_SUCCESS;
     }
 
-    /* get configuration */
-    conf = (py_config *) ap_get_module_config(req->per_dir_config,
-                                              &python_module);
-
-    /* determine interpreter to use */
-    interp_name = select_interp_name(req, NULL, conf, NULL, NULL);
-
-    /* get/create interpreter */
-    idata = get_interpreter(interp_name);
-
-    if (!idata) {
-        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, req,
-                      "handle_python: Can't get/create interpreter.");
-
-        Py_XDECREF(tagobject);
-        Py_XDECREF(codeobject);
-        return HTTP_INTERNAL_SERVER_ERROR;
-    }
-
     /* create/acquire request object */
     request_obj = python_get_request_object(req, 0);
 
@@ -2423,8 +2413,10 @@ static const char *directive_PythonOption(cmd_parms *cmd, void * mconfig,
 
 static const char *directive_PythonOptimize(cmd_parms *cmd, void *mconfig,
                                             int val) {
+#if PY_VERSION_HEX < 0x030C0000
     if ((val) && (Py_OptimizeFlag != 2))
         Py_OptimizeFlag = 2;
+#endif
     return NULL;
 }
 
@@ -2648,7 +2640,12 @@ static void PythonChildInitHandler(apr_pool_t *p, server_rec *s)
 
     /* accordig Py C Docs we must do this after forking */
     PyEval_RestoreThread(global_tstate);
+
+#if PY_VERSION_HEX < 0x03070000
     PyOS_AfterFork();
+#elif PY_VERSION_HEX < 0x030D0000
+    PyOS_AfterFork_Child();
+#endif
 
     interpreterdata *idata = save_interpreter(MAIN_INTERPRETER, PyThreadState_Get());
     if (!idata)
@@ -2992,11 +2989,3 @@ module python_module =
     python_commands,               /* command table */
     python_register_hooks          /* register hooks */
 };
-
-
-
-
-
-
-
-
